package com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.web;

import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.core.persistence.Page;
import com.jeeplus.core.web.BaseController;
import com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.entity.History;
import com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.entity.NormativeAudit;
import com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.service.NormativeAuditService;
import com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.util.ExcelExportUtil;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 储备项目规范性审核记录表
 * @author: 王强
 * @create: 2022-08-23 11:03
 **/
@Controller
@RequestMapping(value = "${adminPath}/cg/reservemanagementcenter/normative")
public class NormativeAuditController extends BaseController {

    @Autowired
    private NormativeAuditService auditService;

    @ModelAttribute
    public NormativeAudit get(@RequestParam(required = false) String id) {
        NormativeAudit entity = null;
        if (StringUtils.isNotBlank(id)) {
            entity = auditService.get(id);
        }
        if (entity == null) {
            entity = new NormativeAudit();
        }
        return entity;
    }

    /**
     * 储备项目规范性审核记录表页面
     * @param normativeAudit
     * @param model
     * @return
     */
    @RequiresPermissions("cg:reservemanagementcenter:normative:list")
    @RequestMapping(value = "normativeList")
    public String normativeList(NormativeAudit normativeAudit, Model model) {
        model.addAttribute("normativeAudit", normativeAudit);
        return "modules/cg/statisticalTable/normative/normative";
    }

    /**
     * 特殊物资统计数据列表
     * @param normativeAudit
     * @param request
     * @param response
     * @param model
     * @return
     */
    @ResponseBody
    @RequiresPermissions("cg:reservemanagementcenter:normative:list")
    @RequestMapping(value = "list")
    public Map<String, Object> basicData(NormativeAudit normativeAudit, HttpServletRequest request, HttpServletResponse response, Model model) {

        Page<NormativeAudit> page = auditService.findPage(new Page<NormativeAudit>(request, response), normativeAudit);
        List<NormativeAudit> list = page.getList();
        List<NormativeAudit> auditList = new ArrayList<>();
        NormativeAudit audit = null;
        for (int i=0;i<list.size();i++){
            audit = list.get(i);
            //进行赋值操作
            //根据projectReserveId查出流程表的proc_ins_id
            String procInsId = auditService.getByProjectReserveId(audit.getProjectReserveId());
            //根据procInsId查出历史流程名称和时间
            List<History> histories = auditService.getHistoryByExecuteId(procInsId);
            //当size>0时待办有历史流程，
            if (histories.size()>0){
                //根据proc_ins_id去运行时任务节点表（act_ru_task）中查出当前流程名称
                String name = auditService.getByExecuteId(procInsId);
                audit.setReserveProgress(name);
            }
            //设置完整性校核相关值
            audit.setBlueprint(audit.getBlueprint() ==null ? "" : audit.getBlueprint());
            audit.setBudgetBook(audit.getBudgetBook() ==null ? "" : audit.getBudgetBook());
            audit.setDesignManual(audit.getDesignManual() ==null ? "" : audit.getDesignManual());

            audit.setProjectProposal(audit.getProjectProposal() ==null ? "" : audit.getProjectProposal());
            //设置拆旧清册
            int count = auditService.getCountByProjectReserveId(audit.getProjectReserveId());
            audit.setOldInventoryDemolition(count >0 ? "" : null);
            //设置技经驳回意见
            audit.setReviewMaterial(audit.getReviewMaterial() == null ? "是" : "否(" + audit.getReviewMaterial() + ")");
            audit.setaSupplyTob(audit.getaSupplyTob() == null ? "是" : "否(" + audit.getaSupplyTob() + ")");
            audit.setTemplateCorrect(audit.getTemplateCorrect() == null ? "是" : "否(" + audit.getTemplateCorrect() + ")");
            audit.setCorrectApplication(audit.getCorrectApplication() == null ? "是" : "否(" + audit.getCorrectApplication() + ")");
            audit.setWorkloadConsistent(audit.getWorkloadConsistent() == null ? "是" : "否(" + audit.getWorkloadConsistent() + ")");
            audit.setFeeCorrect(audit.getFeeCorrect() == null ? "是" : "否(" + audit.getFeeCorrect() + ")");
            audit.setMaterialSupplyA(audit.getMaterialSupplyA() == null ? "是" : "否(" + audit.getMaterialSupplyA() + ")");
            audit.setMaterialSupplyB(audit.getMaterialSupplyB() == null ? "是" : "否(" + audit.getMaterialSupplyB() + ")");
            audit.setChargeLiveWork(audit.getChargeLiveWork() == null ? "是" : "否(" + audit.getChargeLiveWork() + ")");
            audit.setChargeDemolitionWork(audit.getChargeDemolitionWork() == null ? "是" : "否(" + audit.getChargeDemolitionWork() + ")");
            audit.setOtherProblem(audit.getOtherProblem() == null ? "是" : "否(" + audit.getOtherProblem() + ")");

            auditList.add(audit);
        }
        page.setList(auditList);
        return getBootstrapData(page);
    }

    //导出excel
    @ResponseBody
    @RequiresPermissions("cg:reservemanagementcenter:normative:export")
    @RequestMapping(value = "export")
    public void exportExcel(NormativeAudit normativeAudit, HttpServletRequest request, HttpServletResponse response) {
        List<NormativeAudit> list = auditService.exportFindList(normativeAudit);

        List<NormativeAudit> auditList = new ArrayList<>();
        NormativeAudit audit = null;
        for (int i=0;i<list.size();i++){
            audit = list.get(i);
            //进行赋值操作
            //根据projectReserveId查出流程表的proc_ins_id
            String procInsId = auditService.getByProjectReserveId(audit.getProjectReserveId());
            //根据procInsId查出历史流程名称和时间
            List<History> histories = auditService.getHistoryByExecuteId(procInsId);
            //当size>0时待办有历史流程，
            if (histories.size()>0){
                //根据proc_ins_id去运行时任务节点表（act_ru_task）中查出当前流程名称
                String name = auditService.getByExecuteId(procInsId);
                audit.setReserveProgress(name);
            }
            //设置完整性校核相关值
            audit.setBlueprint(audit.getBlueprint() ==null ? "" : audit.getBlueprint());
            audit.setBudgetBook(audit.getBudgetBook() ==null ? "" : audit.getBudgetBook());
            audit.setDesignManual(audit.getDesignManual() ==null ? "" : audit.getDesignManual());

            audit.setProjectProposal(audit.getProjectProposal() ==null ? "" : audit.getProjectProposal());
            //设置拆旧清册
            int count = auditService.getCountByProjectReserveId(audit.getProjectReserveId());
            audit.setOldInventoryDemolition(count >0 ? "" : null);
            //设置技经驳回意见
            audit.setReviewMaterial(audit.getReviewMaterial() == null ? "是" : "否(" + audit.getReviewMaterial() + ")");
            audit.setaSupplyTob(audit.getaSupplyTob() == null ? "是" : "否(" + audit.getaSupplyTob() + ")");
            audit.setTemplateCorrect(audit.getTemplateCorrect() == null ? "是" : "否(" + audit.getTemplateCorrect() + ")");
            audit.setCorrectApplication(audit.getCorrectApplication() == null ? "是" : "否(" + audit.getCorrectApplication() + ")");
            audit.setWorkloadConsistent(audit.getWorkloadConsistent() == null ? "是" : "否(" + audit.getWorkloadConsistent() + ")");
            audit.setFeeCorrect(audit.getFeeCorrect() == null ? "是" : "否(" + audit.getFeeCorrect() + ")");
            audit.setMaterialSupplyA(audit.getMaterialSupplyA() == null ? "是" : "否(" + audit.getMaterialSupplyA() + ")");
            audit.setMaterialSupplyB(audit.getMaterialSupplyB() == null ? "是" : "否(" + audit.getMaterialSupplyB() + ")");
            audit.setChargeLiveWork(audit.getChargeLiveWork() == null ? "是" : "否(" + audit.getChargeLiveWork() + ")");
            audit.setChargeDemolitionWork(audit.getChargeDemolitionWork() == null ? "是" : "否(" + audit.getChargeDemolitionWork() + ")");
            audit.setOtherProblem(audit.getOtherProblem() == null ? "是" : "否(" + audit.getOtherProblem() + ")");

            auditList.add(audit);
        }

        //生成序列号
        int i = 1;
        for (NormativeAudit s : auditList) {
            s.setId(String.valueOf(i));
            i++;
        }

        ExcelExportUtil util = ExcelExportUtil.getOne();
        XSSFWorkbook workbook = util.getWorkbook("normative");
        util.exportNormative(response,workbook,auditList);
    }

}
