package com.jeeplus.modules.sg.balancedlibrary.transitMaterialGoods.web;

import com.google.common.collect.Lists;
import com.jeeplus.common.json.AjaxJson;
import com.jeeplus.common.utils.DateUtils;
import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.common.utils.excel.ExportExcel;
import com.jeeplus.common.utils.excel.ImportExcel;
import com.jeeplus.core.persistence.Page;
import com.jeeplus.core.web.BaseController;
import com.jeeplus.modules.sg.balancedlibrary.materialQuota.entity.MaterialQuota;
import com.jeeplus.modules.sg.balancedlibrary.materialQuota.service.MaterialQuotaService;
import com.jeeplus.modules.sg.balancedlibrary.materialReportDetails.utils.ExcelUtil;
import com.jeeplus.modules.sg.balancedlibrary.materialReportDetails.utils.ExportUtil;
import com.jeeplus.modules.sg.balancedlibrary.onPassageMaterials.entity.MaterialInformation;
import com.jeeplus.modules.sg.balancedlibrary.preferredMaterials.service.PreferredMaterialsService;
import com.jeeplus.modules.sg.balancedlibrary.standardMaterials.entity.StandardMaterials;
import com.jeeplus.modules.sg.balancedlibrary.standardMaterials.service.StandardMaterialsService;
import com.jeeplus.modules.sg.balancedlibrary.transitMaterialGoods.entity.GoodsInfo;
import com.jeeplus.modules.sg.balancedlibrary.transitMaterialGoods.entity.TransitMaterialGoods;
import com.jeeplus.modules.sg.balancedlibrary.transitMaterialGoods.service.TransitMaterialGoodsService;
import com.jeeplus.modules.sg.financial.erpcredit.util.ValidationUtils;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 在途物资Controller
 * @author: 王强
 * @create: 2022-08-30 14:18
 **/
@Controller
@RequestMapping(value = "${adminPath}/transitMaterialGoods")
public class TransitMaterialGoodsController extends BaseController {

    @Autowired
    private TransitMaterialGoodsService service;

    @Autowired
    private StandardMaterialsService standardMaterialsService;

    @Autowired
    private PreferredMaterialsService preferredMaterialsService;

    @Autowired
    private MaterialQuotaService materialQuotaService;

    @ModelAttribute
    public TransitMaterialGoods get(@RequestParam(required = false) String id) {
        TransitMaterialGoods entity = null;
        if (StringUtils.isNotBlank(id)) {
            entity = service.get(id);
        }
        if (entity == null) {
            entity = new TransitMaterialGoods();
        }
        return entity;
    }

    /**
     * 物料替换列表页面
     */
    @RequiresPermissions("sg:balancedlibrary:transitMaterialGoods:list")
    @RequestMapping(value = "list")
    public String list(TransitMaterialGoods entity, Model model) {
        model.addAttribute("entity", entity);
        return "modules/sg/balancedlibrary/transitMaterialGoods/transitMaterialGoodsList";
    }

    /**
     * 标准物资列表数据
     */
    @ResponseBody
    @RequiresPermissions("sg:balancedlibrary:transitMaterialGoods:list")
    @RequestMapping(value = "data")
    public Map<String, Object> data(TransitMaterialGoods entity, HttpServletRequest request, HttpServletResponse response, Model model) {
        Page<TransitMaterialGoods> page = service.findPage(new Page<TransitMaterialGoods>(request, response), entity);
        return getBootstrapData(page);
    }

    /**
     * 在途物资-导入Excel数据
     */
    @ResponseBody
    @PostMapping("import")
    public AjaxJson importFile(@RequestParam("file") MultipartFile[] file, HttpServletResponse response, HttpServletRequest request) {
        AjaxJson ajaxJson = new AjaxJson();

        try {
            //获取两张excel表
            ImportExcel onPassageMaterialsFile = new ImportExcel(file[0], 1, 0);
//            ImportExcel materialInformationFile = new ImportExcel(file[1], 1, 0);
            //转换为集合
            List<TransitMaterialGoods> onPassageMaterialsList = onPassageMaterialsFile.getDataList(TransitMaterialGoods.class);
            service.insertList(onPassageMaterialsList);
//            List<MaterialGoodsInfo> materialInformationList = materialInformationFile.getDataList(MaterialGoodsInfo.class);

//            service.comparativeData(onPassageMaterialsList,materialInformationList);

            /***********************************************/
//            String fileName = "项目物资需求表" + DateUtils.getDate("yyyyMMddHHmmss") + ".xlsx";
//            new ExportExcel("", MaterialInformation.class).setDataList(materialInformationList).write(response, fileName).dispose();
        } catch (Exception e) {
            e.printStackTrace();
        }
        return ajaxJson;
    }

    /**
     * 批量删除在途物资
     */
    @ResponseBody
    @RequiresPermissions("sg:balancedlibrary:transitMaterialGoods:del")
    @RequestMapping(value = "deleteAll")
    public AjaxJson deleteAll(String ids) {
        AjaxJson j = new AjaxJson();
        String idArr[] =ids.split(",");
        List<String> idList = Arrays.asList(idArr);
        service.deleteList(idList);
        j.setMsg("删除标准物资成功");
        return j;
    }

    /**
     * 导出excel文件
     */
    @ResponseBody
    @RequiresPermissions("sg:balancedlibrary:transitMaterialGoods:export")
    @RequestMapping(value = "export")
    public AjaxJson exportFile(TransitMaterialGoods entity, HttpServletRequest request, HttpServletResponse response) {
        AjaxJson j = new AjaxJson();
        try {
            String fileName = "在途物资统计表"+DateUtils.getDate("yyyyMMddHHmmss")+".xlsx";
            Page<TransitMaterialGoods> page = service.findPage(new Page<TransitMaterialGoods>(request, response, -1), entity);
            new ExportExcel("", GoodsInfo.class).setDataList(page.getList()).write(response, fileName).dispose();
            j.setSuccess(true);
            j.setMsg("导出成功！");
            return j;
        } catch (Exception e) {
            j.setSuccess(false);
            j.setMsg("导出在途物资统计表失败！失败信息："+e.getMessage());
        }
        return j;
    }

    /**
     * 下载在途物资统计数据模板
     */
    @ResponseBody
    @RequiresPermissions("sg:balancedlibrary:transitMaterialGoods:import")
    @RequestMapping(value = "import/template")
    public AjaxJson importFileTemplate(HttpServletResponse response) {
        AjaxJson j = new AjaxJson();
        try {
            String fileName = "在途物资统数据导入模板.xlsx";
            List<TransitMaterialGoods> list = Lists.newArrayList();
            new ExportExcel("", TransitMaterialGoods.class, 1).setDataList(list).write(response, fileName).dispose();
            return null;
        } catch (Exception e) {
            j.setSuccess(false);
            j.setMsg( "导入模板下载失败！失败信息："+e.getMessage());
        }
        return j;
    }

    /**
     * 下载物资上报明细导入模板 新的直接从模板文件中获取
     */
    @ResponseBody
    @RequestMapping(value = "import/templateNew")
    public void  importFileTemplateNew(HttpServletResponse response) throws IOException {
        String fileName = "项目物资需求.xlsx";
        String path = this.getClass().getResource("/").getPath()+ "freemarker/excelmodel/requiredMaterialModel.xlsx";
        File file = new File(path);
        InputStream inputStream = new FileInputStream(file);// 将excel文件转为输入流
        XSSFWorkbook workbook = new XSSFWorkbook(inputStream);// 创建个workbook，
        ExportUtil.exportFile(response,fileName,workbook);
    }

    /**
     * 对项目需求物资做验证
     */
    @ResponseBody
    @RequestMapping(value = "importVerify")
    public AjaxJson importFileAndVerify(@RequestParam("file") MultipartFile file, HttpServletResponse response, HttpServletRequest request) throws IOException, InvalidFormatException {
        AjaxJson j = new AjaxJson();
        try {
            ImportExcel materialInformationFile = new ImportExcel(file, 1, 0);
            //转换为集合
            List<MaterialInformation> materList = materialInformationFile.getDataList(MaterialInformation.class);
            //从数据库中获取所有的提报人与部门
            Map<String, String> reportPersonMap = service.getReportPerson();

            ExcelUtil util = new ExcelUtil(file,1,0);

            List<StandardMaterials> standardList = standardMaterialsService.findAllList(new StandardMaterials());
            List<String> preferredCodes = preferredMaterialsService.findAllCodes();
            List<MaterialQuota> quotaList = materialQuotaService.findAllList(new MaterialQuota());
            List<TransitMaterialGoods> goodsList = service.getAllGoods();
            util.dealVerifyZt(standardList,preferredCodes,quotaList,goodsList,materList,reportPersonMap);

            HttpSession session = request.getSession();
            session.setAttribute(ValidationUtils.REQUIRED_MATERIAL_VERIFY,file.getOriginalFilename());
            util.saveToServer(session.getId());
//
//            ValidationUtils.storeObjToRedis(request,workbook,ValidationUtils.REQUIRED_MATERIAL_VERIFY);

            j.setSuccess(true);
            j.setMsg("导入成功，马上下载！");
            return j;
        } catch (Exception e) {
            j.setSuccess(false);
            j.setMsg("导入项目需求物资失败！失败信息：" + e.getMessage());
        }
        return j;


    }

    @ResponseBody
    @RequestMapping(value = "exportVerify")
    public void exportVerify(HttpServletResponse response, HttpServletRequest request) throws IOException, InvalidFormatException {
//        ExcelUtil util = new ExcelUtil(file,1,0);

//        Workbook workbook = ValidationUtils.getSessionRedisObj(request,ValidationUtils.REQUIRED_MATERIAL_VERIFY,Workbook.class);
        HttpSession session = request.getSession();
        String  fileName = (String)session.getAttribute(ValidationUtils.REQUIRED_MATERIAL_VERIFY);
        ExcelUtil util = new ExcelUtil();
        util.exportFile(response,fileName,session.getId());



    }

}
